/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var Q = require('q');
var fs = require('fs');
var path = require('path');

var externalizedStrings = require('./externalizedStrings');
var errors = require('mdo-errors');
var strings = require('ibm-strings');
var exec = require('mdo-exec').run;

/**
 * actually do the work of the command
 */
module.exports.run = function() {
  var deferred = Q.defer();

  // Check for pom.xml
  if (!fs.existsSync('pom.xml') || !fs.existsSync(path.join('src', 'main', 'adapter-resources', 'adapter.xml'))) {
    var nonAdapterError = errors.getError('NONADAPTERFOLDER');
    strings.fatal(nonAdapterError.description);
    deferred.reject(nonAdapterError);
  }
  else {
    console.log(externalizedStrings.buildingAdapter);
    // Run mvn clean install
    exec('mvn', ['clean', 'install']).then(function() {
      console.log(externalizedStrings.successfulBuild);
      deferred.resolve();
    }, function(err) {
      console.log(externalizedStrings.errorBuild);
      console.log(err.stdOut);
      var errorBuild = errors.getError('ADAPTERBUILDERROR');
      strings.fatal(errorBuild.description);
      deferred.reject(errorBuild);
    });
  }

  return deferred.promise;
};

// return the path to the md file (relative to basePath) for the specific command
module.exports.getHelpFilePath = function(basePath) {
  return path.join(basePath, 'mdo-adapter-actions', 'doc', 'build.txt');
};
